package sun.security.krb5;

import sun.security.krb5.internal.Krb5;
import sun.security.krb5.internal.ccache.CCacheOutputStream;
import sun.security.util.DerInputStream;
import sun.security.util.DerOutputStream;
import java.io.IOException;
import sun.security.util.DerValue;
import javax.crypto.spec.DESedeKeySpec;
import javax.crypto.spec.DESKeySpec;
import java.security.GeneralSecurityException;
import sun.security.krb5.internal.crypto.Aes256;
import sun.security.krb5.internal.crypto.Aes128;
import sun.security.krb5.internal.crypto.ArcFourHmac;
import sun.security.krb5.internal.crypto.Des3;
import sun.security.krb5.internal.crypto.Des;
import java.util.Arrays;
import sun.security.krb5.internal.crypto.EType;
import sun.security.krb5.internal.PAData;
import sun.security.krb5.internal.ktab.KeyTab;

public class EncryptionKey implements Cloneable
{
    public static final EncryptionKey NULL_KEY;
    private int keyType;
    private byte[] keyValue;
    private Integer kvno;
    private static final boolean DEBUG;
    
    public synchronized int getEType() {
        return this.keyType;
    }
    
    public final Integer getKeyVersionNumber() {
        return this.kvno;
    }
    
    public final byte[] getBytes() {
        return this.keyValue;
    }
    
    public synchronized Object clone() {
        return new EncryptionKey(this.keyValue, this.keyType, this.kvno);
    }
    
    public static EncryptionKey[] acquireSecretKeys(PrincipalName service, String s) {
        if (service == null) {
            throw new IllegalArgumentException("Cannot have null pricipal name to look in keytab.");
        }
        return KeyTab.getInstance(s).readServiceKeys(service);
    }
    
    public static EncryptionKey acquireSecretKey(PrincipalName principalName, char[] array, int n, PAData.SaltAndParams saltAndParams) throws KrbException {
        String salt;
        byte[] params;
        if (saltAndParams != null) {
            salt = ((saltAndParams.salt != null) ? saltAndParams.salt : principalName.getSalt());
            params = saltAndParams.params;
        }
        else {
            salt = principalName.getSalt();
            params = null;
        }
        return acquireSecretKey(array, salt, n, params);
    }
    
    public static EncryptionKey acquireSecretKey(char[] array, String s, int n, byte[] array2) throws KrbException {
        return new EncryptionKey(stringToKey(array, s, array2, n), n, null);
    }
    
    public static EncryptionKey[] acquireSecretKeys(char[] array, String s) throws KrbException {
        int[] defaults = EType.getDefaults("default_tkt_enctypes");
        EncryptionKey[] array2 = new EncryptionKey[defaults.length];
        for (int i = 0; i < defaults.length; ++i) {
            if (EType.isSupported(defaults[i])) {
                array2[i] = new EncryptionKey(stringToKey(array, s, null, defaults[i]), defaults[i], null);
            }
            else if (EncryptionKey.DEBUG) {
                System.out.println("Encryption Type " + EType.toString(defaults[i]) + " is not supported/enabled");
            }
        }
        return array2;
    }
    
    public EncryptionKey(byte[] array, int keyType, Integer kvno) {
        if (array != null) {
            System.arraycopy(array, 0, this.keyValue = new byte[array.length], 0, array.length);
            this.keyType = keyType;
            this.kvno = kvno;
            return;
        }
        throw new IllegalArgumentException("EncryptionKey: Key bytes cannot be null!");
    }
    
    public EncryptionKey(int n, byte[] array) {
        this(array, n, null);
    }
    
    private static byte[] stringToKey(char[] password, String s, byte[] array, int type) throws KrbCryptoException {
        char[] charArray = s.toCharArray();
        char[] a = new char[password.length + charArray.length];
        System.arraycopy(password, 0, a, 0, password.length);
        System.arraycopy(charArray, 0, a, password.length, charArray.length);
        Arrays.fill(charArray, '0');
        try {
            switch (type) {
                case 1:
                case 3: {
                    return Des.string_to_key_bytes(a);
                }
                case 16: {
                    return Des3.stringToKey(a);
                }
                case 23: {
                    return ArcFourHmac.stringToKey(password);
                }
                case 17: {
                    return Aes128.stringToKey(password, s, array);
                }
                case 18: {
                    return Aes256.stringToKey(password, s, array);
                }
                default: {
                    throw new IllegalArgumentException("encryption type " + EType.toString(type) + " not supported");
                }
            }
        }
        catch (GeneralSecurityException cause) {
            KrbCryptoException ex = new KrbCryptoException(cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
        finally {
            Arrays.fill(a, '0');
        }
    }
    
    public EncryptionKey(char[] array, String s, String s2) throws KrbCryptoException {
        if (s2 == null || s2.equalsIgnoreCase("DES")) {
            this.keyType = 3;
        }
        else if (s2.equalsIgnoreCase("DESede")) {
            this.keyType = 16;
        }
        else if (s2.equalsIgnoreCase("AES128")) {
            this.keyType = 17;
        }
        else if (s2.equalsIgnoreCase("ArcFourHmac")) {
            this.keyType = 23;
        }
        else if (s2.equalsIgnoreCase("AES256")) {
            this.keyType = 18;
            if (!EType.isSupported(this.keyType)) {
                throw new IllegalArgumentException("Algorithm " + s2 + " not enabled");
            }
        }
        else {
            throw new IllegalArgumentException("Algorithm " + s2 + " not supported");
        }
        this.keyValue = stringToKey(array, s, null, this.keyType);
        this.kvno = null;
    }
    
    public EncryptionKey(EncryptionKey encryptionKey) throws KrbCryptoException {
        this.keyValue = Confounder.bytes(encryptionKey.keyValue.length);
        for (int i = 0; i < this.keyValue.length; ++i) {
            byte[] keyValue = this.keyValue;
            int n = i;
            keyValue[n] ^= encryptionKey.keyValue[i];
        }
        this.keyType = encryptionKey.keyType;
        try {
            if (this.keyType == 3 || this.keyType == 1) {
                if (!DESKeySpec.isParityAdjusted(this.keyValue, 0)) {
                    this.keyValue = Des.set_parity(this.keyValue);
                }
                if (DESKeySpec.isWeak(this.keyValue, 0)) {
                    this.keyValue[7] ^= 0xF0;
                }
            }
            if (this.keyType == 16) {
                if (!DESedeKeySpec.isParityAdjusted(this.keyValue, 0)) {
                    this.keyValue = Des3.parityFix(this.keyValue);
                }
                byte[] key = new byte[8];
                for (int j = 0; j < this.keyValue.length; j += 8) {
                    System.arraycopy(this.keyValue, j, key, 0, 8);
                    if (DESKeySpec.isWeak(key, 0)) {
                        this.keyValue[j + 7] ^= 0xF0;
                    }
                }
            }
        }
        catch (GeneralSecurityException cause) {
            KrbCryptoException ex = new KrbCryptoException(cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
    }
    
    public EncryptionKey(DerValue derValue) throws Asn1Exception, IOException {
        if (derValue.getTag() != 48) {
            throw new Asn1Exception(906);
        }
        DerValue derValue2 = derValue.getData().getDerValue();
        if ((derValue2.getTag() & 0x1F) == 0x0) {
            this.keyType = derValue2.getData().getBigInteger().intValue();
            DerValue derValue3 = derValue.getData().getDerValue();
            if ((derValue3.getTag() & 0x1F) == 0x1) {
                this.keyValue = derValue3.getData().getOctetString();
                if (derValue3.getData().available() > 0) {
                    throw new Asn1Exception(906);
                }
                return;
            }
            else {
                throw new Asn1Exception(906);
            }
        }
        else {
            throw new Asn1Exception(906);
        }
    }
    
    public synchronized byte[] asn1Encode() throws Asn1Exception, IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        DerOutputStream derOutputStream2 = new DerOutputStream();
        derOutputStream2.putInteger(this.keyType);
        derOutputStream.write(DerValue.createTag((byte)(-128), true, (byte)0), derOutputStream2);
        DerOutputStream derOutputStream3 = new DerOutputStream();
        derOutputStream3.putOctetString(this.keyValue);
        derOutputStream.write(DerValue.createTag((byte)(-128), true, (byte)1), derOutputStream3);
        DerOutputStream derOutputStream4 = new DerOutputStream();
        derOutputStream4.write((byte)48, derOutputStream);
        return derOutputStream4.toByteArray();
    }
    
    public synchronized void destroy() {
        if (this.keyValue != null) {
            for (int i = 0; i < this.keyValue.length; ++i) {
                this.keyValue[i] = 0;
            }
        }
    }
    
    public static EncryptionKey parse(DerInputStream derInputStream, byte b, boolean b2) throws Asn1Exception, IOException {
        if (b2 && ((byte)derInputStream.peekByte() & 0x1F) != b) {
            return null;
        }
        DerValue derValue = derInputStream.getDerValue();
        if (b != (derValue.getTag() & 0x1F)) {
            throw new Asn1Exception(906);
        }
        return new EncryptionKey(derValue.getData().getDerValue());
    }
    
    public synchronized void writeKey(CCacheOutputStream cCacheOutputStream) throws IOException {
        cCacheOutputStream.write16(this.keyType);
        cCacheOutputStream.write16(this.keyType);
        cCacheOutputStream.write32(this.keyValue.length);
        for (int i = 0; i < this.keyValue.length; ++i) {
            cCacheOutputStream.write8((int)this.keyValue[i]);
        }
    }
    
    @Override
    public String toString() {
        return new String("EncryptionKey: keyType=" + this.keyType + " kvno=" + this.kvno + " keyValue (hex dump)=" + ((this.keyValue == null || this.keyValue.length == 0) ? " Empty Key" : (10 + Krb5.hexDumper.encodeBuffer(this.keyValue) + 10)));
    }
    
    public static EncryptionKey findKey(int n, EncryptionKey[] array) throws KrbException {
        return findKey(n, null, array);
    }
    
    private static boolean versionMatches(Integer n, Integer obj) {
        return n == null || (int)n == 0 || obj == null || (int)obj == 0 || n.equals(obj);
    }
    
    public static EncryptionKey findKey(int n, Integer n2, EncryptionKey[] array) throws KrbException {
        if (!EType.isSupported(n)) {
            throw new KrbException("Encryption type " + EType.toString(n) + " is not supported/enabled");
        }
        boolean n3 = false;
        int n4 = 0;
        Object o = null;
        for (int i = 0; i < array.length; ++i) {
            int eType = array[i].getEType();
            if (EType.isSupported(eType)) {
                Integer keyVersionNumber = array[i].getKeyVersionNumber();
                if (n == eType) {
                    n3 = true;
                    if (versionMatches(n2, keyVersionNumber)) {
                        return array[i];
                    }
                    if ((int)keyVersionNumber > n4) {
                        o = array[i];
                        n4 = (int)keyVersionNumber;
                    }
                }
            }
        }
        if (n == 1 || n == 3) {
            for (int j = 0; j < array.length; ++j) {
                int eType2 = array[j].getEType();
                if (eType2 == 1 || eType2 == 3) {
                    Integer keyVersionNumber2 = array[j].getKeyVersionNumber();
                    n3 = true;
                    if (versionMatches(n2, keyVersionNumber2)) {
                        return new EncryptionKey(n, array[j].getBytes());
                    }
                    if ((int)keyVersionNumber2 > n4) {
                        o = new EncryptionKey(n, array[j].getBytes());
                        n4 = (int)keyVersionNumber2;
                    }
                }
            }
        }
        if (!(n3 == 0)) {
            return o;
        }
        return null;
    }
    
    static {
        NULL_KEY = new EncryptionKey(new byte[0], 0, null);
        DEBUG = Krb5.DEBUG;
    }
}
